/**********************************************************
						WARPOFDMRX.C
WARPOFDM is the driver that allows user code to manipulate
states of both the OFDM transmitter and receiver.
Additionally, it allows direct control of certain behaviour
of PHY.

Author: Chris Hunter 5/25/06
***********************************************************/

#include "xparameters.h"
#include "ofdm_TxRx_mimo.h"

//********Globals********
unsigned char interruptType = 0x00;//No interrupts by default; user code must enabled them later

//********Receiver********
void mimo_ofdmRx_init(){
	/*
		-Disables all packet detection & clears state
		-Holds master reset high
	*/

	*((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_RxReg_ControlBits_OFFSET)) = RX_GLOBAL_RESET;
	*((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Rx_PktDone_Reset_OFFSET)) = 0x33;
}


void mimo_ofdmRx_enable(){
	/*
		-Enables packet detection of type INTERNALDETECTION or EXTERNALDETECTION
		-Toggles rxPktDoneReset_address
		-Lowers master reset
	*/

	//Enable detection type
	unsigned int x = ofdm_TxRx_mimo_ReadReg_RxReg_ControlBits(OFDM_BASEADDR);
	ofdm_TxRx_mimo_WriteReg_RxReg_ControlBits(OFDM_BASEADDR, (x & ~RX_GLOBAL_RESET));
}

void mimo_ofdmRx_disable(){
	/*
		-Disables packet detection
		-Holds rxPktDoneReset_address in reset
	*/


	unsigned int x = ofdm_TxRx_mimo_ReadReg_RxReg_ControlBits(OFDM_BASEADDR);
	ofdm_TxRx_mimo_WriteReg_RxReg_ControlBits(OFDM_BASEADDR, (x | RX_GLOBAL_RESET));
}

void mimo_ofdmRx_setOptions(unsigned int someOptions, unsigned int intType){
	/*
		-Sets options
	*/

//	interruptType = 0x30 & (someOptions >> 12);			//store interrupt type (good and/or bad) to global
//	someOptions = someOptions & 0x00000FFF;		//clear interrupt types from someOptions

	//Assumes warpofdmrx.h has:
	//#define INTR_BAD_PKTS		0x20
	//#define INTR_GOOD_PKTS	0x10

	interruptType = 0x30 & (intType);			//store interrupt type (good and/or bad) to global
	*((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_RxReg_ControlBits_OFFSET)) = someOptions;//Enable options
	*((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Rx_PktDone_Reset_OFFSET)) = interruptType; //Configure the user-selected interrupts; interruptType can only be 0x00, 0x10, 0x20, or 0x30
}


void mimo_ofdmRx_getOptions(){
	/*
		-Reads out control bits
	*/

	xil_printf("Options are     0x%x\r\n",*((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_RxReg_ControlBits_OFFSET)));
	xil_printf("Interrupts are  0x%x\r\n",*((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_RxReg_ControlBits_OFFSET))>>4);
}



//********MAC-PHY Interface Commands********


void mimo_mac2phy_PktAck(){
	//Assert the gloabl reset in the Rx PHY
	//*((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Rx_GlobalReset_OFFSET)) = 0x1;

	/*
		-Acknowledges a packet (good or bad)
	*/
	*((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Rx_PktDone_Reset_OFFSET)) = interruptType | 0x3;	//Raise interrupt reset
	*((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Rx_PktDone_Reset_OFFSET)) = interruptType; 			//Lower interrupt reset

	//Deassert the gloabl reset in the Rx PHY
	//*((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Rx_GlobalReset_OFFSET)) = 0x0;
}


