/**********************************************************
						WARPOFDMRX.H
WARPOFDM is the driver that allows user code to manipulate
states of both the OFDM transmitter and receiver.
Additionally, it allows direct control of certain behaviour
of PHY.

Author: Chris Hunter 5/25/06
***********************************************************/

#ifndef WARPOFDMRX_H
#define WARPOFDMRX_H

#define OFDM_BASEADDR XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR


//********Prototypes********
void mimo_ofdmRx_init();
void mimo_ofdmRx_enable();
void mimo_ofdmRx_disable();
void mimo_ofdmRx_setOptions(unsigned int someOptions, unsigned int intType);
void mimo_ofdmRx_getOptions();
void mimo_mac2phy_PktAck();

//********Defines********

//Control Options

//Rx Control Options
#define RESET_BER 			0x1
#define REQ_LONG_CORR		0x2
#define DYNAMC_PKT_LENGTHS	0x4
#define DYNAMIC_MOD			0x8
#define RX_SISO_MODE		0x10
#define REQ_TWO_LONG_CORR	0x20
#define REQ_SHORT_CORR		0x40
#define EXT_PKT_DETECT 		0x80
#define INT_PKT_DETECT 		0x100
#define BYPASS_CARR_REC		0x200
#define CFO_USE_LTS			0x400
#define CFO_USE_STS			0x800
#define CFO_USE_LONGCORR	0x1000
#define USE_PILOT_ARCTAN		0x2000
#define EQ_BYPASS_DIVISION  0x4000
#define CFO_USE_TRAINING	0x8000

#define RX_GLOBAL_RESET		0x80000000

#define INTR_BAD_PKTS		0x20
#define INTR_GOOD_PKTS		0x10



//Macros
#define mimo_ofdmRx_setByteNums(c) ofdm_TxRx_mimo_WriteReg(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR, ofdm_TxRx_mimo_Rx_pktByteNums_OFFSET, c)

#define mimo_ofdmRx_setFFTScaling(c) ofdm_TxRx_mimo_WriteReg_TxRx_FFT_Scaling(OFDM_BASEADDR, ((ofdm_TxRx_mimo_ReadReg_TxRx_FFT_Scaling(OFDM_BASEADDR) & 0x0003F) | ( (c&0x3F)<<6)))

#define mimo_ofdmRx_setRxScaling(c) *((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Rx_Constellation_Scaling_OFFSET)) = c

#define mimo_ofdmRx_setLongCorrParams(c) *((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Rx_PktDet_LongCorr_Params_OFFSET)) = c
#define mimo_ofdmRx_setNumOFDMSyms(c) *((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Rx_OFDM_SymbolCounts_OFFSET))= c

//#define mimo_ofdmRx_setCorrThresh(c) *((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Rx_pktDet_Corr_Thresh_OFFSET)) = c
//#define mimo_ofdmRx_setEnergyThresh(c) *((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Rx_pktDet_Energy_Thresh_OFFSET)) = c


#define mimo_ofdmRx_copyHeader(dest,len) memcpy((dest),XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_SMRO_RxPktBuffer_OFFSET,(len))
#define mimo_ofdmRx_copyPayload(dest,len) memcpy((dest),XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_SMRO_RxPktBuffer_OFFSET+6*sizeof(int),(len))

#define mimo_ofdmRx_setFFTWindowOffset(c) ofdm_TxRx_mimo_WriteReg(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR, ofdm_TxRx_mimo_Rx_PktDet_Delay_OFFSET, (ofdm_TxRx_mimo_ReadReg(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR, ofdm_TxRx_mimo_Rx_PktDet_Delay_OFFSET) & 0xFFFFE07F) | ((c&0x3F)<<7))
#define mimo_ofdmRx_setPktDetDly(c) ofdm_TxRx_mimo_WriteReg(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR, ofdm_TxRx_mimo_Rx_PktDet_Delay_OFFSET, (ofdm_TxRx_mimo_ReadReg(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR, ofdm_TxRx_mimo_Rx_PktDet_Delay_OFFSET) & 0xFFFFFF80)| (c&0x7F))

//#define mimo_ofdmRx_setCFO_A_KP(c) ofdm_TxRx_mimo_WriteReg_Rx_FreqOffFilt_A_KP(OFDM_BASEADDR, c)

#define mimo_ofdmRx_setCFO_B_KI(c) ofdm_TxRx_mimo_WriteReg_Rx_FreqOffFilt_B_KI(OFDM_BASEADDR, c)
#define mimo_ofdmRx_setCFO_B_KP(c) ofdm_TxRx_mimo_WriteReg_Rx_FreqOffFilt_B_KP(OFDM_BASEADDR, c)

#define mimo_ofdmRx_setPNTrack_K(c) ofdm_TxRx_mimo_WriteReg_Rx_PhaseNoiseTrack_K(OFDM_BASEADDR, c)
#define mimo_ofdmRx_setPNTrack_Kalpha(c) ofdm_TxRx_mimo_WriteReg_Rx_PhaseNoiseTrack_Kalpha(OFDM_BASEADDR, c)
#define mimo_ofdmRx_setPNTrack_Kbeta(c) ofdm_TxRx_mimo_WriteReg_Rx_PhaseNoiseTrack_Kbeta(OFDM_BASEADDR, c)
#define mimo_ofdmRx_setPNTrack_Kgamma(c) ofdm_TxRx_mimo_WriteReg_Rx_PhaseNoiseTrack_Kgamma(OFDM_BASEADDR, c)

#define mimo_ofdmRx_setCorrThresh(c) ofdm_TxRx_mimo_WriteReg_Rx_pktDet_Tresholds(OFDM_BASEADDR, ((ofdm_TxRx_mimo_ReadReg_Rx_pktDet_Tresholds(OFDM_BASEADDR) & 0xFFFFFF00) | (c&0xFF)))
#define mimo_ofdmRx_setEnergyThresh(c) ofdm_TxRx_mimo_WriteReg_Rx_pktDet_Tresholds(OFDM_BASEADDR, ((ofdm_TxRx_mimo_ReadReg_Rx_pktDet_Tresholds(OFDM_BASEADDR) & 0xFFF000FF) | ((c&0xFFF)<<12)))


#endif
