#include "ofdm_AGC_mimo.h"
#include "xparameters.h"

void ofdm_AGC_SetDCO(unsigned int AGCstate){

	// Enables DCO and DCO subtraction (correction scheme and butterworth hipass are active)

	unsigned int bits;
	bits = *((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_BitsIn_OFFSET));

	if(AGCstate)
		bits = bits | 0x6;
	else
		bits = bits & 0x1;

	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_BitsIn_OFFSET) ) = bits;
	xil_printf("AGC BITSIN = %x\r\n",*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_BitsIn_OFFSET) ));
}

void ofdm_AGC_FiltSel(unsigned int AGCstate){

	// Sets the downsampler AGCstate (0 is DS, 1 is filtered)

	unsigned int bits;
	bits = *((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_BitsIn_OFFSET));

	if(AGCstate)
		bits = bits | 0x1;
	else
		bits = bits & 0x6;

	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_BitsIn_OFFSET)) = bits;

}

void ofdm_AGC_Reset(){

	// Cycle the agc's software reset port

	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_SRESET_IN_OFFSET) ) =1;
	usleep(10);
	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_SRESET_IN_OFFSET) ) =0;
	usleep(100);

}


void ofdm_AGC_MasterReset(){

	// Cycle the master reset register in the AGC and enable it

	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_AGC_EN_OFFSET)) = 0;
	usleep(10);

	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_MRESET_IN_OFFSET)) = 0;
	usleep(10);
	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_MRESET_IN_OFFSET)) = 1;
	usleep(10);
	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_MRESET_IN_OFFSET)) = 0;
	usleep(10);

	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_AGC_EN_OFFSET)) = 1;

}

void ofdm_AGC_Initialize(int noise_estimate){

	int g_bbset = 0;

	// First set all standard parameters

	// Turn off both resets and the master enable
	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_AGC_EN_OFFSET)) = 0;
	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_SRESET_IN_OFFSET) ) = 0;
	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_MRESET_IN_OFFSET)) = 0;


	// An adjustment parameter
	*((volatile signed int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_ADJ_OFFSET)) = 8;

	// Timing for the DC-offset correction
	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_DCOTiming_OFFSET)) = 0x46403003;

	// Initial baseband gain setting
	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_GBB_init_OFFSET)) = 52;

	// RF gain AGCstate thresholds
	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_Thresholds_OFFSET)) = 0xD5CBA6;//0xE0D3A6;

	// Overall AGC timing
	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_Timing_OFFSET)) = 0x9A962A28;//0x826E3C0A;

	// vIQ and RSSI average lengths
	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_AVG_LEN_OFFSET)) = 0x10F; //103

	// Disable DCO, disable DCO subtraction, set filter to straight downsampling
	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_BitsIn_OFFSET) ) = 0x0;

	// Compute and set the initial g_BB gain value from the noise estimate
	// The initial g_bb sets noise to -19 db, assuming 32 db RF gain

	g_bbset = -19 - 32 - noise_estimate;
	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_GBB_init_OFFSET)) = g_bbset;


	// Perform a master reset
	ofdm_AGC_MasterReset();

	// Agc is now reset and enabled, ready to go!
}


void ofdm_AGC_setNoiseEstimate(int noise_estimate){
	int g_bbset;
	g_bbset = -19 - 32 - noise_estimate;
	*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_GBB_init_OFFSET)) = g_bbset;
	xil_printf("-19 - 32 - noise_estimate = %d\r\n",*((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_GBB_init_OFFSET)));

}

unsigned int ofdm_AGC_GetGains(void){

	// Returns hi[0 gBB_B g_RF_B | 0 g_BB_A g_RF_A]lo

	unsigned int gBB_A, gRF_A, gBB_B, gRF_B, gains;

	// Get the gains from the registers

	gBB_A = *((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_G_BB_A_reg_OFFSET));
	gRF_A = *((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_G_RF_A_reg_OFFSET));


	gBB_B = *((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_G_BB_B_reg_OFFSET));
	gRF_B = *((volatile unsigned int *)(XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_G_RF_B_reg_OFFSET));

	// Concatenate into one return register

	// First concatenate the two radios together, into the gRF register
	// 2 lowest bits are RF, 5 higher bits are BB, last bit is unused

	// Multiply by 2^2, shift gBB right by 2 bits

	gRF_A = gRF_A + (gBB_A * 4);
	gRF_B = gRF_B + (gBB_B * 4);

	// Multiply by 2^8 shift gRF right by 8 bits

	gains = gRF_A + (gRF_B * 256);

	return gains;


}

void ofdm_AGC_SetTarget(unsigned int target){
	*((volatile unsigned int*) (XPAR_OFDM_AGC_MIMO_OPBW_0_BASEADDR+OFDM_AGC_MIMO_T_dB_OFFSET)) = target;
	return;
}
