//WARPMAC Framework
//v2.1
/***************CHANGELOG*****************
+updated radio controller to 1.08.a		CRH
******************************************/
/*****************WARPMAC*****************
Description: This framework allows for custom
MAC implementations on WARP. For a description
of the functions, see:
http://warp.rice.edu/trac/wiki/WARPMAC
******************************************/

#ifndef WARPMAC_H
#define WARPMAC_H

#include "xparameters.h"
#include "radio_controller_basic.h"
#include "radio_controller_ext.h"
#include "radio_controller_adv.h"
#include <pthread.h>
#include "xemac_l.h"
#include "xemac.h"
#include <xstatus.h>
#include "xintc.h"
#include "xutil.h"
#include "xgpio.h"
#include "xexception_l.h"


#define FIRST_DAC DAC2_ADDR
#define SECOND_DAC DAC1_ADDR
#define FIRST_RADIO RADIO2_ADDR
#define SECOND_RADIO RADIO1_ADDR

//LED - Parameters that set up "Knight Rider" functionality
#define LEDLEN 6
#define BLINKEVERY 1 //Currently, LED switches position every 20 received ACKs

#define MY_XEM_MAX_FRAME_SIZE 2000
#define MY_XEM_MAX_FRAME_SIZE_IN_WORDS ((MY_XEM_MAX_FRAME_SIZE/sizeof(Xuint32))+1)




//Threads
#define N_THREADS 1	//1 thread: ackmac_main()

//Interrupts
#define TMA_INTERRUPT 3 //position in the interrupt vector inside intc peripheral
#define RX_INTERRUPT 2
#define BADRX_INTERRUPT 1
#define PB_INTERRUPT 0


//Masks for configuring modulation settings in the PHY
//Each is 8 copies of a 4-bit modulation value
#define MODMASK_BPSK 0x11111111
#define MODMASK_QPSK 0x22222222
#define MODMASK_16QAM 0x44444444
#define MODMASK_64QAM 0x66666666

//PHY Interface
//#define RATE 2 //Moved definition to tx init in warpmac.c

//Timers
#define TIMEOUT 0
#define BACKOFF 1
#define SLOTTIME 10
//#define TIMEOUTTIME 1000 //This is the bare minimum (checked with scope) do not lower  500

typedef struct {
	volatile unsigned char sequenceNumber[1];
} Macsequence;

//Frame Structure
//Note: This frame structure is used internally by the MAC. While it contains much of the same
//information, this is not to be confused with the char array frame that is sent to the PHY.
typedef struct {
	volatile unsigned char pktRev;
	volatile unsigned char pktType;
	volatile unsigned char destAddr[6];
	volatile unsigned char srcAddr[6];
	volatile unsigned char currReSend;
	volatile unsigned int length;
	//volatile unsigned char *data;
   volatile Xuint32 *data __attribute__ ((aligned (8)));
	volatile unsigned int chkSum;
	volatile unsigned char isNew;
} Macframe;





//Control Structure
//This is a struct of miscellaneous control bits needed for correct operation of the ACKMAC.
typedef struct {
	volatile unsigned int timeout;
	volatile unsigned int slotTime;
	volatile unsigned char selfAddr[6];
	volatile unsigned char maxReSend;
	volatile unsigned char currBackoff;
	volatile unsigned char maxBackoff;	//maxBackoff is the index, starting at 0, of the number of times the backoff window doubles
	volatile unsigned char timerType;
	volatile unsigned char timerEnabled;
	volatile unsigned int currBackoffTime;
	volatile unsigned char enableSequencing;
	volatile unsigned char packetDupe;
	volatile unsigned char enableEthernetInt;
	Macsequence mySequenceNum;
	Macsequence sequenceVec[256];

} Maccontrol;





//Function Prototypes
void warpmac_CalibrateTxDCO();
unsigned char sevenSegmentMap(unsigned char x);
void warpmac_incrementLED();
void warpmac_incrementLEDLow();
void warpmac_incrementLEDHigh();
void timer_a_int_handler(void * baseaddr_p);
void rxPhyBad(void *baseaddr_p);
void rxPhyGood(void *baseaddr_p);
void pb_int_handler(void *baseaddr_p);
int warpmac_clearTimer();
void warpmac_setTimerVal(Xuint32 clocks);
void warpmac_startTimer(unsigned char mode);
int randNum(int N);
unsigned int warpmac_carrierSense();
int warpmac_setTimer(int type);
int warpmac_init();
void warpmac_sendEthernet(Macframe* packet);
int warpmac_sendOfdm(Macframe* packet);
void warpmac_setUpButtonHandler(void(*handler)());
void warpmac_setLeftButtonHandler(void(*handler)());
void warpmac_setRightButtonHandler(void(*handler)());
void warpmac_setMiddleButtonHandler(void(*handler)());
void warpmac_setTimerHandler(void(*handler)());
void warpmac_setGoodPacketHandler(void(*handler)());
void warpmac_setBadPacketHandler(void(*handler)());
void warpmac_setEmacHandler(void(*handler)());
int warpmac_incrementResend(Macframe* packet);
int warpmac_allocatePayload(Macframe* packet,unsigned int length);
int warpmac_freePayload(Macframe* packet);
int warpmac_addressedToMe(Macframe* packet);
void warpmac_setMaxResend(unsigned int c);
void warpmac_setMaxCW(unsigned int c);
void warpmac_setChannel(unsigned int c);
int warpmac_setMacAddr(unsigned char* addr);
int warpmac_setTimeout(unsigned int time);
int warpmac_setSlotTime(unsigned int time);
int warpmac_inTimeout();
int warpmac_getMyId();
int warpmac_copyPayload(Macframe* packet);
void warpmac_CalibrateTxDCO ();
void warpmac_enableSisoMode();
void warpmac_enableMimoMode();
void warpmac_setNumTrainingSyms(unsigned int c);
unsigned char sevenSegmentMap(unsigned char x);
void warpmac_enableCSMA();
void warpmac_disableCSMA();
void warpmac_enableSequencing();
void warpmac_disableSequencing();
void warpmac_setPktDlyPlus();
void warpmac_setPktDlyMinus();
void warpmac_setKPPlus(unsigned int increment);
void warpmac_setKPMinus(unsigned int decrement);
void warpmac_setKIPlus(unsigned int increment);
void warpmac_setKIMinus(unsigned int decrement);
void warpmac_setNoiseTargetPlus();
void warpmac_setNoiseTargetMinus();
void warpmac_setTargetPlus();
void warpmac_setTargetMinus();
void warpmac_leftHex(unsigned char x);
void warpmac_rightHex(unsigned char x);
int warpmac_addressedToThem(Macframe* packet,unsigned char* addr);
int warpmac_addressedFromThem(Macframe* packet,unsigned char* addr);
void warpmac_disableDetection();
void warpmac_enableDetection();
void warpmac_set_modulation(unsigned char baseRate, unsigned char antAFullRate, unsigned char antBFullRate);
void warpmac_phyInterruptClear();
static void emacTx_int_handler(void *Callback);
static void emacRx_int_handler(void *Callback);
static void emacErr_int_handler(void *Callback, XStatus ErrCode);
void warpmac_enableEthernetInterrupt();
void warpmac_disableEthernetInterrupt();

#endif


